/* 
 * The FeedUpdateBroker class implements a simple RSS fetcher and parser.
 * 
 */

// Constructor.
function FeedUpdateBroker() {
    this.httpReq = null;
    this.callback = null;
	this.useCacheWorkaround = true;
	this.DATETIME_MATCHER = /(\d\d\d\d)-(\d\d?)-(\d\d?)T(\d\d?):(\d\d?):(\d\d?)/
}
FeedUpdateBroker.prototype.fetchFeed = function(feedURL, callback, ns) {
    this.callback = callback;
    this.httpReq = new AjaxThing();
    var self = this;
    this.httpReq.onreadystatechange = function() { self.readyStateChanged(ns); };
	var fullURL = feedURL;
	if (this.useCacheWorkaround) {
		if (fullURL.indexOf("?") == -1) {
			fullURL += "?";
		} else {
			fullURL += "&";
		}
		fullURL += "nocache=" + (new Date().getTime());
	}
    debug("sending request: GET ");
	showloading();
    this.httpReq.open("GET", fullURL, true);
    this.httpReq.send(null);
}
FeedUpdateBroker.prototype.readyStateChanged = function(ns) {
    debug("readyStateChanged "+this.httpReq.readyState);
    if (this.httpReq.readyState == 4) {
        var responseStatus = null;
        try {
            responseStatus = this.httpReq.status;
        } catch (noStatusException) {}
        this.callback.call(this, this.handleResponse(responseStatus, this.httpReq.responseXML, ns));
    }
}
FeedUpdateBroker.prototype.parseElement= function(e) {
	var items = new Array();
	for (var i = 0; i < e.length; i++) {
		var item = new Array();
        var node = e[i].firstChild;
        while (node != null) {
			if (node.childNodes.length > 1) {
				item[node.nodeName]=[];
				item[node.nodeName]=this.parseElement(node.childNodes);
			} else if (node.nodeType == Node.ELEMENT_NODE) {
                if (node.nodeName == "date") {
					item['date']=this.getDateOfNode(node);
				} else {
                    item[node.nodeName] = this.getTextOfNode(node);
                }
            }
            node = node.nextSibling;
        }
        // create the item and add to the items array
		items.push(item);
	}
	return items;
}

// Handles a completed response.
FeedUpdateBroker.prototype.handleResponse = function(responseStatus, xmlDoc, ns) {
    if (responseStatus == 200 && xmlDoc != null) {
        // node ref for iterating
        var node;
        // get last modified time - default to the zero
        var lastModified = null;
        // init feed items array
        var items = [];
        // we got the feed XML so now we'll parse it
        var itemElements = xmlDoc.getElementsByTagName(ns);
		items=this.parseElement(itemElements);
        // update was completed successfully
        return { status: "ok", lastModified: lastModified, items: items };
    } else {
        // update failed
        return { status: "error" };
    }
}
// Returns the date as a Date object from a node representing a date
FeedUpdateBroker.prototype.getDateOfNode = function(node) {
	var dateString = this.getTextOfNode(node);
	var parsedDate = new Date(0);

	var matches = this.DATETIME_MATCHER.exec(dateString);
	if (!matches) {
		var parsedTimestamp = Date.parse(dateString);
		if (!isNaN(parsedTimestamp)) {
			parsedDate.setTime(parsedTimestamp);
		}
		else {
			return null;
		}
	}
	else {
		// the matches is an array containing the numerical parts of the string '2009-01-01T22:00:00Z'
		parsedDate.setFullYear(parseInt(matches[1]), parseInt(matches[2]) - 1, parseInt(matches[3]));
		// without setting the microseconds, the old value would be left
		parsedDate.setHours(parseInt(matches[4]), parseInt(matches[5]), parseInt(matches[6]), 0);
	}
	//alert(Helper.formatDate(parsedDate));
	return parsedDate;
}
// Returns the text of a node.
FeedUpdateBroker.prototype.getTextOfNode = function(node) {
    var buf = "";
    
    // iterate through all child elements and collect all text to the buffer
    var child = node.firstChild;
    while (child != null) {
        if (child.nodeType == Node.TEXT_NODE || child.nodeType == Node.CDATA_SECTION_NODE) {
            // append text to buffer
            if (buf != "") {
                buf += " ";
            }
            buf += child.nodeValue;
        }
        child = child.nextSibling;
    }
    
    // strip all tags from the buffer
    var strippedBuf = "";
    var textStartPos = -1;
    var tagBalance = 0;
    
    // iterate through the text and append all text to the stripped buffer
    // that is at a tag balance of 0
    for (pos = 0; pos < buf.length; pos++) {
        var c = buf.charAt(pos);
        if (c == '<') {
            // entering a tag
            if (tagBalance == 0 && textStartPos != -1) {
                // everything up to here was valid text
                strippedBuf += buf.substring(textStartPos, pos);
                textStartPos = -1;
            }
            tagBalance++;
        } else if (c == '>') {
            // leaving a tag
            tagBalance--;
            textStartPos = -1;
        } else if (tagBalance == 0 && textStartPos == -1) {
            // first char of text
            textStartPos = pos;
        }
    }
    
    // add remaining text - if any
    if (tagBalance == 0 && textStartPos != -1) {
        strippedBuf += buf.substring(textStartPos, pos);
    }
    
    return strippedBuf;
}
